@@ -1,6 +1,6 @@
-/* $Id: ifp_routines.c,v 1.55 2005/05/08 03:54:32 yamajun Exp $ */
+/* $Id: ifp_routines.c,v 1.59 2005/12/03 16:55:06 yamajun Exp $ */
 
-static const char rcsid[] = "$Id: ifp_routines.c,v 1.55 2005/05/08 03:54:32 yamajun Exp $";
+static const char rcsid[] = "$Id: ifp_routines.c,v 1.59 2005/12/03 16:55:06 yamajun Exp $";
 
 #include <stdio.h>
 #include <string.h>
@@ -80,7 +80,7 @@ void remove_double_slash(char *path, size_t size);
 void slash2backslash(char *dest, const char *src, size_t count);
 int ifp_ctl_datasize(unsigned char *control);
 char *dump_control_regs(const char *control, int length);
-int safe_suffix(const char *name);
+int safe_suffix(usb_dev_handle *dh, const char *name);
 int firmware_check_ifp(const char *firm_name);
 int firmware_check_n10(const char *firm_name);
 int ifp_file_exists(usb_dev_handle *dh, const char *name);
@@ -155,10 +155,6 @@ int ifp_upload(usb_dev_handle *dh, int argc, char *argv[]) {
 	    if (ifp_isdir(dh, ifp_path)) {
 		// file to dir
 		snprintf(ifp_path, IFP_MAXPATHLEN, "%s/%s", argv[2], get_basename(argv[1]));
-		//strncpy(ifp_path, argv[2], IFP_MAXPATHLEN - 2);
-		//strncat(ifp_path, "/", 1);
-		//strncat(ifp_path, get_basename(argv[1]),
-		//	IFP_MAXPATHLEN - strlen(ifp_path) - 1);
 		normalize_path(ifp_path, IFP_MAXPATHLEN);
 
 		retval = ifp_upload_file(dh, IS_IFP, ifp_path, fp);
@@ -296,7 +292,7 @@ int ifp_download(usb_dev_handle *dh, int argc, char *argv[]) {
 	    break;
 	}
 
-	if (!safe_suffix(argv[2])) {
+	if (!safe_suffix(dh, argv[2])) {
 	    break;
 	}
 
@@ -317,7 +313,7 @@ int ifp_download(usb_dev_handle *dh, int argc, char *argv[]) {
 	switch (getc(stdin)) {
 	case 'Y':	/* yes */
 	case 'y':
-	    if (!safe_suffix(argv[2])) {
+	    if (!safe_suffix(dh, argv[2])) {
 		break;
 	    }
 
@@ -333,18 +329,12 @@ int ifp_download(usb_dev_handle *dh, int argc, char *argv[]) {
 	if (ifp_isdir(dh, ifp_path)) {
 	    // dir to dir
 	    snprintf(path_buf, PATH_MAX, "%s/%s", argv[2], get_basename(ifp_path));
-	    //strncpy(path_buf, argv[2], PATH_MAX - 2);
-	    //strncat(path_buf, "/", 1);
-	    //strncat(path_buf, get_basename(ifp_path), PATH_MAX - strlen(path_buf) - 1);
 	    retval = ifp_download_dir(dh, ifp_path, path_buf);
 
 	} else {
 	    // file to dir
 	    snprintf(path_buf, PATH_MAX, "%s/%s", argv[2], get_basename(ifp_path));
-	    //strncpy(path_buf, argv[2], PATH_MAX - 2);
-	    //strncat(path_buf, "/", 1);
-	    //strncat(path_buf, get_basename(ifp_path), PATH_MAX - strlen(path_buf) - 1);
-	    if (!safe_suffix(path_buf)) {
+	    if (!safe_suffix(dh, path_buf)) {
 		break;
 	    }
 	    retval = ifp_download_file(dh, IS_IFP, ifp_path, path_buf);
@@ -545,8 +535,8 @@ int ifp_list_dir_printf(usb_dev_handle *dh, const char *name, int format, int re
 	    goto LS_END;
 	case 1:
 	    usb_bulk_read(dh, IFP_BULK_FROM, lsbuf, sizeof(lsbuf), TOUT);
+	    unicode2locale(ascii_lsbuf, IFP_LSBUFLEN, lsbuf, IFP_LSBUFLEN);
 
-	    unicode2string(ascii_lsbuf, lsbuf);
 	    strncpy(filename, ifp_path, IFP_MAXPATHLEN);
 	    if (strcmp(ifp_path, "/") != 0) {
 		strcat(filename, "/");
@@ -583,7 +573,7 @@ int ifp_list_dir_printf(usb_dev_handle *dh, const char *name, int format, int re
 	    break;
 	case 2:
 	    usb_bulk_read(dh, IFP_BULK_FROM, lsbuf, sizeof(lsbuf), TOUT);
-	    unicode2string(ascii_lsbuf, lsbuf);
+	    unicode2locale(ascii_lsbuf, IFP_LSBUFLEN, lsbuf, IFP_LSBUFLEN);
 
 	    strncpy(filename, ifp_path, IFP_MAXPATHLEN);
 	    if (strcmp(filename, "/") != 0) strcat(filename, "/");
@@ -694,6 +684,7 @@ int ifp_check_connect(usb_dev_handle *dh) {
 }
 
 int ifp_format(usb_dev_handle *dh) {
+    struct usb_device *dev = usb_device(dh);
     unsigned char ctl[4];
 
     printf("WARNING: Do you want to format iFP? [y/N]: ");
@@ -707,7 +698,40 @@ int ifp_format(usb_dev_handle *dh) {
     }
 
     printf("Formating. Please wait...\n");
+    // send control message
     usb_control_msg(dh, IFP_REQ_TYPE, IFP_FORMAT, 0, 0, ctl, sizeof(ctl), TOUT);
+    // FIXME
+    //printf("Done.\n");
+
+    //return 0;
+    //usb_control_msg(dh, IFP_REQ_TYPE, IFP_FIRMWARE_UPDATE, 0, 0, ctl, sizeof(ctl), TOUT);
+
+    // close/reopen iFP for update.
+    usb_release_interface(dh, dev->config->interface->altsetting->bInterfaceNumber);
+
+//#ifdef linux 
+//    usb_reset(dh);	// Don't use for Mac OS X.
+//#endif
+
+    usb_close(dh);
+    sleep(FIRMUPDATE_TIMEOUT);	// need this!
+
+    if ( (dh = usb_open(dev)) == NULL) {
+	fprintf(stderr, "Unable to open iFP device.\n");
+	return -1;
+    }
+
+    if (usb_claim_interface(dh, dev->config->interface->altsetting->bInterfaceNumber)) {
+	fprintf(stderr, "Device is busy.  (I was unable to claim its interface.)\n");
+	usb_close(dh);
+	exit(1);
+    }
+
+    /*if (!ifp_check_connect(dh)) {
+	fprintf(stderr, "Unable to connect to iFP.\n");
+	return -1;
+    }*/
+
     printf("Done.\n");
 
     return 0;
@@ -930,9 +954,6 @@ int ifp_upload_dir(usb_dev_handle *dh, const char *from, const char *name) {
 
 	// "path" "file" -> "path/file"
 	snprintf(ifp_path, IFP_MAXPATHLEN, "%s/%s", name, toname);
-	//strncpy(ifp_path, name, IFP_MAXPATHLEN - 2);
-	//strncat(ifp_path, "/", 1);
-	//strncat(ifp_path, toname, IFP_MAXPATHLEN - strlen(ifp_path) - 1);
 
 	switch (ftsentp->fts_info) {
 	case FTS_F:		// normal file
@@ -1012,22 +1033,15 @@ int ifp_download_dir(usb_dev_handle *dh, const char *fromdir, const char *todir)
 	    break;
 	case 1:	// file
 	    usb_bulk_read(dh,IFP_BULK_FROM,inbuf,sizeof(inbuf),TOUT);
-	    unicode2string(strbuf,inbuf);
+	    unicode2locale(strbuf, IFP_LSBUFLEN, inbuf, IFP_LSBUFLEN);
 
 	    // "path" "file" -> "path/file"
 	    snprintf(ifp_path, IFP_MAXPATHLEN, "%s/%s", fromdir, strbuf);
-	    //strncpy(ifp_path, fromdir, IFP_MAXPATHLEN - 2);
-	    //strncat(ifp_path, "/", 1);
-	    //strncat(ifp_path, strbuf, IFP_MAXPATHLEN - strlen(ifp_path) - 1);
-
 	    snprintf(to_path, IFP_MAXPATHLEN, "%s/%s", todir, strbuf);
-	    //strncpy(to_path, todir, IFP_MAXPATHLEN - 2);
-	    //strncat(to_path, "/", 1);
-	    //strncat(to_path, strbuf, IFP_MAXPATHLEN - strlen(to_path) - 1);
 
 	    switch (filestat(to_path)) {
 	    case 0:	// "to_path" not found;
-		if (!safe_suffix(to_path)) {
+		if (!safe_suffix(dh, to_path)) {
 		    break;
 		}
 
@@ -1038,7 +1052,7 @@ int ifp_download_dir(usb_dev_handle *dh, const char *fromdir, const char *todir)
 		switch (getc(stdin)) {
 		case 'Y':	/* yes */
 		case 'y':
-		    if (!safe_suffix(to_path)) {
+		    if (!safe_suffix(dh, to_path)) {
 			break;
 		    }
 
@@ -1060,13 +1074,10 @@ int ifp_download_dir(usb_dev_handle *dh, const char *fromdir, const char *todir)
 	    break;
 	case 2:	// directory
 	    usb_bulk_read(dh,IFP_BULK_FROM,inbuf,sizeof(inbuf),TOUT);
-	    unicode2string(strbuf,inbuf);
+	    unicode2locale(strbuf, IFP_LSBUFLEN, inbuf, IFP_LSBUFLEN);
 
 	    // "path" "file" -> "path/file"
 	    snprintf(ifp_path, IFP_MAXPATHLEN, "%s/%s", fromdir, strbuf);
-	    //strncpy(ifp_path, fromdir, IFP_MAXPATHLEN - 2);
-	    //strncat(ifp_path, "/", 1);
-	    //strncat(ifp_path, get_basename(strbuf), IFP_MAXPATHLEN - strlen(ifp_path) - 1);
 	    retval = push_dir(ifp_path);
 
 	    break;
@@ -1085,9 +1096,6 @@ LS_END:
     while (pop_dir(strbuf)) {
 	// "path" "file" -> "path/file"
 	snprintf(to_path, IFP_MAXPATHLEN, "%s/%s", todir, strbuf);
-	//strncpy(to_path, todir, IFP_MAXPATHLEN - 2);
-	//strncat(to_path, "/", 1);
-	//strncat(to_path, get_basename(strbuf), IFP_MAXPATHLEN - strlen(to_path) - 1);
 
 	retval = ifp_download_dir(dh, strbuf, to_path);
 	if (retval == -1) {
@@ -1139,26 +1147,20 @@ int ifp_delete_recursive(usb_dev_handle *dh, char *name) {
 
 	case 1:	// file
 	    usb_bulk_read(dh,IFP_BULK_FROM,inbuf,sizeof(inbuf),TOUT);
-	    unicode2string(strbuf,inbuf);
+	    unicode2locale(strbuf, IFP_LSBUFLEN, inbuf, IFP_LSBUFLEN);
 
 	    // "path" "file" -> "path/file"
 	    snprintf(ifp_path, IFP_MAXPATHLEN, "%s/%s", name, strbuf);
-	    //strncpy(ifp_path, name, IFP_MAXPATHLEN - 2);
-	    //strncat(ifp_path, "/", 1);
-	    //strncat(ifp_path, strbuf, IFP_MAXPATHLEN - strlen(ifp_path) - 1);
 	    retval = ifp_delete_file(dh, ifp_path);
 
 	    break;
 
 	case 2:	// directory
 	    usb_bulk_read(dh,IFP_BULK_FROM,inbuf,sizeof(inbuf),TOUT);
-	    unicode2string(strbuf,inbuf);
+	    unicode2locale(strbuf, IFP_LSBUFLEN, inbuf, IFP_LSBUFLEN);
 
 	    // "path" "file" -> "path/file"
 	    snprintf(ifp_path, IFP_MAXPATHLEN, "%s/%s", name, strbuf);
-	    //strncpy(ifp_path, name, IFP_MAXPATHLEN - 2);
-	    //strncat(ifp_path, "/", 1);
-	    //strncat(ifp_path, strbuf, IFP_MAXPATHLEN - strlen(ifp_path) - 1);
 	    retval = push_dir(ifp_path);
 
 	    break;
@@ -1258,8 +1260,9 @@ int ifp_ok(unsigned char *control) {
 int ifp_send_filename(usb_dev_handle *dh, const char *name) {
     unsigned char buf[IFP_BULK_MAXPATHLEN];
     unsigned char strbuf[IFP_MAXPATHLEN];
+
     slash2backslash(strbuf, name, strlen(name)+1);
-    string2unicode(buf,strbuf);
+    locale2unicode(buf, IFP_BULK_MAXPATHLEN ,strbuf, IFP_MAXPATHLEN);
     return ifp_send_data(dh,buf,sizeof(buf));
 }
 
@@ -1546,7 +1549,8 @@ int ifp_file_exists(usb_dev_handle *dh, const char *name) {
 	case 1:				
 	    //if (type != 1) break;
 	    usb_bulk_read(dh,IFP_BULK_FROM,lsbuf,sizeof(lsbuf),TOUT);
-	    unicode2string(ascii_lsbuf,lsbuf);
+	    unicode2locale(ascii_lsbuf, IFP_LSBUFLEN, lsbuf, IFP_LSBUFLEN);
+
 	    if (strcmp(ascii_lsbuf, get_basename(name)) == 0) {
 		retval = 1;	// "name" is exists and it's a file
 		goto LS_END;
@@ -1556,7 +1560,8 @@ int ifp_file_exists(usb_dev_handle *dh, const char *name) {
 	case 2:
 	    //if (type != 2) break;
 	    usb_bulk_read(dh,IFP_BULK_FROM,lsbuf,sizeof(lsbuf),TOUT);
-	    unicode2string(ascii_lsbuf,lsbuf);
+	    //unicode2string(ascii_lsbuf,lsbuf);
+	    unicode2locale(ascii_lsbuf, IFP_LSBUFLEN, lsbuf, IFP_LSBUFLEN);
 	    if (strcmp(ascii_lsbuf, get_basename(name)) == 0) {
 		retval = 2;	// "name" is exists and it's a directory
 		goto LS_END;
@@ -1579,10 +1584,12 @@ LS_END:
     return retval;
 }
 
-int safe_suffix(const char *name) {
-    if ( (ifp_strcasestr(name, ".mp3") || 
-	  ifp_strcasestr(name, ".wma") || 
-	  ifp_strcasestr(name, ".asf")) && isOldIFP) {
+int safe_suffix(usb_dev_handle *dh, const char *name) {
+    if ( (ifp_firmware_version(dh) <= 322 ) && isOldIFP &&
+	  (ifp_strcasestr(name, ".mp3") ||
+	   ifp_strcasestr(name, ".wma") ||
+	   ifp_strcasestr(name, ".asf"))
+	 ) {
 	fprintf(stderr, "%s: this filetype cannot be downloaded.\n", name);
 	return 0;   // false
     }
